# Paolo Mairano, 2008
# Procedura per creare un canvas con un grafico xy con serie formate da un solo dato (x;y).
#
# Uso:
# grafico $nome ?opts?
#
# Le opzioni sono:
# -noffset (default 50) spazio da lasciare a nord del grafico
# -soffset (default 50) spazio da lasciare a sud del grafico
# -woffset (default 50) spazio da lasciare a ovest del grafico
# -eoffset (default 50) spazio da lasciare a est del grafico
# -bg (default white) --- sfondo del grafico
# -contour (default white) --- sfondo del canvas
# -width (default 400) --- larghezza del canvas
# -height (default 400) --- altezza del canvas
# -xtickcol (default black) --- colore dei numeri dell'asse delle x
# -ytickcol (default black) --- colore dei numeri dell'asse delle y
# -xtickfont (default "") --- font dei numeri dell'asse delle x
# -ytickfont (default "") --- font dei numeri dell'asse delle y
# -xlabelcol (default black) --- colore dell'etichetta dell'asse delle x
# -ylabelcol (default black) --- colore dell'etichetta dell'asse delle y
# -xlabelfont (default "") --- font dell'etichetta dell'asse delle x
# -ylabelfont (default "") --- font dell'etichetta dell'asse delle y
# -title (default "") --- titolo
# -titlecol (default black) --- colore del titolo
# -titlefont (default "") --- font del titolo
# -titleoffset (default 20) --- distanza del titolo dal grafico
# -axiswidth (default 1) --- spessore degli assi
# -axiscol (default black) --- colore degli assi
# -xticklinecol (default black) --- colore della griglia
# -xticklinewidth (default 1) --- spessore della griglia
# -yticklinecol (default black) --- colore della griglia
# -yticklinewidth (default 1) --- spessore della griglia
# -tickfont (default "") --- font dei tick
# -xdash (default "") --- dash della griglia x
# -ydash (default "") --- dash della griglia y
# -xoffset (default 5) --- distanza dei tick dall'asse delle x
# -xlabeloffset (default $xoffset*4) --- distanza dell'etichetta dall'asse delle x
# -xmin (default valore minimo) --- valore minimo dell'asse delle x
# -xmax (default valore massimo) --- valore massimo dell'asse delle x
# -xspan (default (xmax-xmin)5/) --- intervallo fra un tick e l'altro dell'asse delle x
# -xlabel (default "") --- etichetta dell'asse delle x
# -xformat (default %.2f) --- formato numero dell'asse delle x
# -xticklines (default 0) --- griglia delle x
# -yoffset (default 7) --- distanza dei tick dall'asse delle y
# -ylabeloffset (default $titleoffset) --- distanza dell'etichetta dell'asse delle y
# -ymin (default calcolato) --- valore minimo dell'asse delle y
# -ymax (default calcolato) --- valore massimo dell'asse delle y
# -yspan (default no) --- intervallo fra un tick e l'altro dell'asse delle y
# -ylabel (default "") --- etichetta dell'asse delle y
# -yformat (default %.2f) --- formato numero dell'asse delle y
# -yticklines (default 0) --- intervallo
#

namespace eval pchart {


proc x {num} {
	variable width
	variable woffset
	variable eoffset
	set pxls_asse [expr $width - ($woffset + $eoffset)]
	variable xmin
	variable xmax
	set scala_asse [expr double($xmax - $xmin)]
	set rapporto [expr $pxls_asse / $scala_asse]
	set ris [expr ($rapporto * ($num - $xmin)) + $woffset]
}

proc y {num} {
	variable height
	variable noffset
	variable soffset
	set pxls_asse [expr $height - ($noffset + $soffset)]
	variable ymin
	variable ymax
	set scala_asse [expr double($ymax - $ymin)]
	set rapporto [expr $pxls_asse / $scala_asse]
	set ris [expr ($rapporto * ($num - $ymin)) - $noffset]
	set invertito [expr $pxls_asse - $ris]
}

proc draw {nome args} {
	variable history ""
	lappend history [concat draw $nome]
	variable canvas $nome
	
	# legge gli argomenti
	foreach {a b} $args {
		variable [string range $a 1 end] $b
	}
	
	# per ogni var, se non esiste dàlle il valore di default
	foreach {var value} {
		width 400
		height 400
		noffsetperc 10
		soffsetperc 10
		woffsetperc 10
		eoffsetperc 10
		xmin 0
		xmax 100
		xspan "[expr ($xmax - $xmin) / 5.0]"
		ymin 0
		ymax 100
		yspan "[expr ($ymax - $ymin) / 5.0]"
		move 1
		tip 1
		bg white
		contour white
		title "{}"
		titlecol black
		titlefont "[font actual TkDefaultFont]"
		titlecoordx 0
		titlecoordy -10
		xtickcol black
		ytickcol black
		xtickfont "[font actual TkDefaultFont]"
		ytickfont "[font actual TkDefaultFont]"
		xaxiswidth 1
		yaxiswidth 1
		xaxiscol black
		yaxiscol black
		xticklinecol black
		yticklinecol black
		xticklinewidth 1
		yticklinewidth 1
		xoffset 5
		yoffset 7
		xlabelcoordx 0
		xlabelcoordy "[expr $xoffset * 4]"
		ylabelcoordx 0
		ylabelcoordy -20
		xlabel x
		xformat "%.0f"
		xticklines 0
		xdash "{}"
		ylabel y
		yformat "%.0f"
		yticklines 0
		ydash "{}"
		xlabelcol black
		ylabelcol black
		xlabelfont "[font actual TkDefaultFont]"
		ylabelfont "[font actual TkDefaultFont]"
	} {
		if [info exists pchart::$var] {variable $var} else [concat variable $var $value]
	}
	
	# calcola gli offset
	variable noffset [perc2px y $noffsetperc]
	variable soffset [perc2px y $soffsetperc]
	variable woffset [perc2px x $woffsetperc]
	variable eoffset [perc2px x $eoffsetperc]
	
	# azzera il contatore di serie inserite
	variable serie 0
	
	# trova i tick
	set xticks ""
	for {set i $xmin} {$i <= $xmax} {set i [expr $i + $xspan]} {
		if {$i == $xmin} {continue}
		lappend xticks $i
	}
	set yticks ""
	for {set i $ymin} {$i <= $ymax} {set i [expr $i + $yspan]} {
		if {$i == $ymin} {continue}
		lappend yticks $i
	}
	
	# crea canvas
	catch {canvas $nome -width $width -height $height -background $contour}
	
	# sfondo
	$nome create rect [x $xmin] [y $ymin] [x $xmax] [y $ymax] -outline $contour -fill $bg -tag unraiseable
	
	# titolo
	$nome create text [expr [x [expr $xmax - (($xmax - $xmin)/2.0)]] + $titlecoordx] [expr [y $ymax] + $titlecoordy] -text $title -fill $titlecol -anchor center -tag {title moveable} -font $titlefont
	if {$move == 1} {
		$canvas bind title <B1-Motion> "pchart::drag title %x %y"
		$canvas bind title <Button-1> {set ::pchart::titlemarkx %x; set ::pchart::titlemarky %y}
	}

	# disegna assi, etichette e origine
	$nome create line [x $xmin] [y $ymin] [x $xmax] [y $ymin] -fill $xaxiscol -width $xaxiswidth -tag xaxis
	$nome create text [x $xmin] [expr [y $ymin] + $xoffset] -text [format $xformat $xmin] -fill $xtickcol -anchor n -tag xtick
	$nome create text [expr [x $xmax] + $xlabelcoordx] [expr [y $ymin] + $xlabelcoordy] -text $xlabel -anchor n -fill $xlabelcol -tag {xlabel moveable} -font $xlabelfont
	$nome create line [x $xmin] [y $ymin] [x $xmin] [y $ymax] -fill $yaxiscol -width $yaxiswidth -tag yaxis
	$nome create text [expr [x $xmin] - $yoffset] [y $ymin] -text [format $yformat $ymin] -fill $ytickcol -anchor e -tag ytick
	$nome create text [expr [x $xmin] + $ylabelcoordx] [expr [y $ymax] + $ylabelcoordy] -text $ylabel -anchor e -fill $ylabelcol -tag {moveable ylabel} -font $ylabelfont
	
	# bindings per il trascinamento
	if {$move == 1} {
		$canvas bind xlabel <B1-Motion> "pchart::drag xlabel %x %y"
		$canvas bind xlabel <Button-1> {set ::pchart::xlabelmarkx %x; set ::pchart::xlabelmarky %y}
		$canvas bind ylabel <B1-Motion> "pchart::drag ylabel %x %y"
		$canvas bind ylabel <Button-1> {set ::pchart::ylabelmarkx %x; set ::pchart::ylabelmarky %y}
	}
	
	# crea tick e griglia (se non ci sono i tick disegna comunque xmax e ymax)
	foreach tick $xticks {
		$nome create line [x $tick] [y $ymin] [x $tick] [expr [y $ymin] + 4] -fill $xaxiscol -width $xaxiswidth -tag xaxis
		$nome create text [x $tick] [expr [y $ymin] + $xoffset] -text [format $xformat $tick] -fill $xtickcol -anchor n -tag xtick
		if {$xticklines eq 1} {$nome create line [x $tick] [y $ymin] [x $tick] [y $ymax] -fill $xticklinecol -width $xticklinewidth -tag xtickline}
	}
	if {$xticks eq ""} {
		$nome create text [x $xmax] [expr [y $ymin] + $xoffset] -text $xmax -fill $xtickcol -anchor n -tag xtick
	}
	foreach tick $yticks {
		$nome create line [x $xmin] [y $tick] [expr [x $xmin] - 4] [y $tick] -fill $yaxiscol -width $yaxiswidth -tag yaxis
		$nome create text [expr [x $xmin] - $yoffset] [y $tick] -text [format $yformat $tick] -fill $ytickcol -anchor e -tag ytick
		if {$yticklines eq 1} {$nome create line [x $xmin] [y $tick] [x $xmax] [y $tick] -fill $yticklinecol -width $yticklinewidth -tag ytickline}
	}
	if {$yticks eq ""} {
		$nome create text [expr [x $xmin] - $yoffset] [y $ymax] -text $ymax -fill $ytickcol -anchor e -tag ytick
	}
	
	# configurazioni
	if {$xdash ne ""} {$nome itemconfigure xtickline -dash $xdash}
	if {$ydash ne ""} {$nome itemconfigure ytickline -dash $ydash}
	$nome itemconfigure xtick -font $xtickfont
	$nome itemconfigure ytick -font $ytickfont
	
	# bindings
	bind $nome <Motion> {pchart::scan %x %y}
	if {$move == 1} {
		bind $nome <ButtonRelease-1> {pchart::raise %x %y}
	}
	
	return $nome
}

proc insertSeries {nome data args} {
	lappend pchart::history "insertSeries $nome {$data} $args"
	variable arr
	variable canvas
	variable serie
	incr serie
	
	#valori di default
	set arr($serie.showinleg) 1
	set arr($serie.fill) white
	set arr($serie.outline) black
	set arr($serie.shape) c
	set arr($serie.size) 4
	set arr($serie.width) 1
	set arr($serie.showlab) 1
	set arr($serie.xbar) "0 0"
	set arr($serie.ybar) "0 0"
	
	# crea un array arr del namespace con tutti gli argomenti (serve per le etichette e la legenda)
	foreach {a b} $args {
		set arr($serie.[string range $a 1 end]) $b
	}
	set arr($serie.name) $nome
	set arr($serie.data) $data
	
	# per ogni punto
	foreach {x y} $arr($serie.data) {
		# crea le coordinate dell'etichetta (se la serie ha più di 1 punto, l'etichetta apparirà sul primo)
		if ![info exists arr($serie.labcoordx)] {set arr($serie.labcoordx) 0}
		if ![info exists arr($serie.labcoordy)] {set arr($serie.labcoordy) [expr 0 - ($arr($serie.size)*2)]}
		
		# se ha valori oltre gli assi ritorna
		if {($x<$::pchart::xmin) | ($x>$::pchart::xmax) | ($y<$::pchart::ymin) | ($y>$::pchart::ymax)} {return}
		
		# disegna le barra d'errore
		if {([lindex $arr($serie.xbar) 0] != 0) || ([lindex $arr($serie.xbar) 1] != 0)} {addErrorbar x $serie}
		if {([lindex $arr($serie.ybar) 0] != 0) || ([lindex $arr($serie.ybar) 1] != 0)} {addErrorbar y $serie}
		
		# disegna l'indicatore con il bind che mostra il nome della serie al passaggio del mouse
		addIndicator [x $x] [y $y] $serie "serie$serie ind"
	}
}

proc addErrorbar {tipo serie} {
	variable canvas
	variable arr
	set x [lindex $arr($serie.data) 0]
	set y [lindex $arr($serie.data) 1]
	set a [lindex $arr($serie.[set tipo]bar) 0]
	set b [lindex $arr($serie.[set tipo]bar) 1]
	set size $arr($serie.size)
	set col $arr($serie.outline)
	set width $arr($serie.width)
	if {$tipo eq "x"} {
		$canvas create line [x [expr $x + $a]] [y $y] [x [expr $x + $b]] [y $y] -fill $col -width $width
		$canvas create line [x [expr $x + $a]] [expr [y $y] + $size] [x [expr $x + $a]] [expr [y $y] - $size] -fill $col -width $width
		$canvas create line [x [expr $x + $b]] [expr [y $y] + $size] [x [expr $x + $b]] [expr [y $y] - $size] -fill $col -width $width
	}
	if {$tipo eq "y"} {
		$canvas create line [x $x] [y [expr $y + $a]] [x $x] [y [expr $y + $b]] -fill $col -width $width
		$canvas create line [expr [x $x] + $size] [y [expr $y + $a]] [expr [x $x] - $size] [y [expr $y + $a]] -fill $col -width $width
		$canvas create line [expr [x $x] + $size] [y [expr $y + $b]] [expr [x $x] - $size] [y [expr $y + $b]] -fill $col -width $width
	}
}

proc addIndicator {x y serie tag} {
	variable canvas
	variable arr
	switch $arr($serie.shape) {
		"c" {return [circle $canvas $x $y $arr($serie.size) $arr($serie.fill) $arr($serie.outline) $arr($serie.width) $tag]}
		"s" {return [square $canvas $x $y $arr($serie.size) $arr($serie.fill) $arr($serie.outline) $arr($serie.width) $tag]}
		"tu" {return [triangleup $canvas $x $y $arr($serie.size) $arr($serie.fill) $arr($serie.outline) $arr($serie.width) $tag]}
		"td" {return [triangledown $canvas $x $y $arr($serie.size) $arr($serie.fill) $arr($serie.outline) $arr($serie.width) $tag]}
		"tl" {return [triangleleft $canvas $x $y $arr($serie.size) $arr($serie.fill) $arr($serie.outline) $arr($serie.width) $tag]}
		"tr" {return [triangleright $canvas $x $y $arr($serie.size) $arr($serie.fill) $arr($serie.outline) $arr($serie.width) $tag]}
		"d" {return [diamond $canvas $x $y $arr($serie.size) $arr($serie.fill) $arr($serie.outline) $arr($serie.width) $tag]}
		"x" {return [cross $canvas $x $y $arr($serie.size) $arr($serie.fill) $arr($serie.width) $tag]}
		"+" {return [plus $canvas $x $y $arr($serie.size) $arr($serie.fill) $arr($serie.width) $tag]}
		"*" {return [asterisk $canvas $x $y $arr($serie.size) $arr($serie.fill) $arr($serie.width) $tag]}
	}
}

proc insertLabels {args} {
	lappend pchart::history [concat insertLabels $args]
	variable serie
	variable arr
	variable canvas
	
	# legge gli argomenti
	foreach {a b} $args {
		variable [string range $a 1 end] $b
	}
	
	# prende i valori di default se alcune proprietà non sono specificate come argomento
	foreach {var value} {
		labfont [font actual TkDefaultFont]
		labbg white
		labfg black
		labbd black
		labborder 1
	} {
		if [info exists pchart::$var] {variable $var} else [concat variable $var $value]
	}
	
	# per ogni serie in arr
	for {set conto 1} {$conto <= $serie} {incr conto} {
		
		# se showlab è no vai al prossimo, altrimenti prendi x e y
		if !$arr($conto.showlab) {continue}
		set x [lindex $arr($conto.data) 0]
		if {($x < $pchart::xmin) || ($x > $pchart::xmax)} {continue} else {set x [x $x]}
		set y [lindex $arr($conto.data) 1]
		if {($y < $pchart::ymin) || ($y > $pchart::ymax)} {continue} else {set y [y $y]}
		
		# se esistono prendi i valori specificati per una determinata serie, se no quelli generali
		foreach prop {labfont labbg labfg labbd labborder} {
			if [info exists arr($conto.$prop)] {set local_$prop $arr($conto.$prop)} else {set local_$prop [set $prop]}
		}
		
		# scrivi l'etichetta
		$canvas create text [expr $x + $arr($conto.labcoordx)] [expr $y + $arr($conto.labcoordy)] -text $arr($conto.name) -anchor s -fill $local_labfg -font $local_labfont -tag "label$conto label"
		
		# disegna il bordo se è richiesto
		if {$local_labborder > 0} {
			set bbox [$canvas bbox "label$conto"]
			$canvas create rect [expr [lindex $bbox 0] - 2] [lindex $bbox 1] [lindex $bbox 2] [lindex $bbox 3] -fill $local_labbg -outline $local_labbd -width $local_labborder -tag "label$conto border label"
			$canvas raise "label$conto && !border"
		}
		
		# crea i binding all'etichetta per spostarla con il trascinamento
		if {$pchart::move == 1} {
			$canvas bind label$conto <B1-Motion> "pchart::dragLabel $conto %x %y"
			$canvas bind label$conto <Button-1> {set ::pchart::labmarkx %x; set ::pchart::labmarky %y}
		}
	}
}

proc insertLegend {args} {
	lappend pchart::history [concat insertLegend $args]
	variable arr
	variable serie
	variable canvas
	
	# legge gli argomenti
	foreach {a b} $args {
		variable [string range $a 1 end] $b
	}
	
	#valori di default
	foreach {var value} {
		legendfont "[font actual TkDefaultFont]"
		legendspace 15
		legendbg white
		legendfg black
		legendbd black
		legendborder 1
		adaptwidth 0
		adaptheight 0
	} {
		if [info exists pchart::$var] {variable $var} else [concat variable $var $value]
	}
	
	# se non esistono le coordinate della legenda trasformale in floating numbers, se non esistono stabiliscile
	if [info exists ::pchart::legx] {variable legx} else {variable legx [expr $::pchart::width - $::pchart::eoffset + 30]}
	if [info exists ::pchart::legy] {variable legy} else {variable legy $::pchart::noffset}
	if [string is integer $legx] {set legx $legx.0}
	if [string is integer $legy] {set legy $legy.0}
	
	# per ogni serie in arr
	set inseriti 0
	for {set conto 1} {$conto <= $serie} {incr conto} {
		
		# se showinleg è 0 passa al prossimo
		if !$arr($conto.showinleg) {continue}		
		
		# trova y
		incr inseriti
		set y [expr $legy + ($legendspace * $inseriti)]
		
		# disegna indicatore
		addIndicator $legx $y $conto "legend serie$conto"
		
		# aggiungi nome serie
		$canvas create text [expr $legx + $arr($conto.size) + 4] $y -text $arr($conto.name) -fill $legendfg -anchor w -font $legendfont -tag "legend serie$conto"
	}
		
	# crea rettangolo e porta il resto in primo piano
	set bbox [$canvas bbox legend]
	if {$legendborder > 0} {
		$canvas create rect [expr [lindex $bbox 0] - 4] [expr [lindex $bbox 1] -2] [expr [lindex $bbox 2] + 2] [expr [lindex $bbox 3] + 2] -fill $legendbg -outline $legendbd -width $legendborder -tag {legend rect}
		$canvas raise {legend && !rect}
	}
	
	# crea i binding alla legenda per spostarla con il trascinamento
	if {$pchart::move == 1} {
		$canvas bind legend <B1-Motion> {pchart::dragLegend %x %y}
		$canvas bind legend <Button-1> {set ::pchart::legmarkx %x; set ::pchart::legmarky %y}
	}
	
	# adatta la larghezza del grafico per far stare la legenda
	if $adaptwidth {adapt width}
	if $adaptheight {adapt height}
	
}

proc drag {tag x y} {
	variable canvas
	
	# ottiene i nomi delle variabili che controllano la posizione dell'oggetto durante il trascinamento
	set varmarkx [subst $tag]markx
	set varmarky [subst $tag]marky
	variable $varmarkx
	variable $varmarky
	
	# calcola la distanza di cui si è mosso il mouse
	set x [$canvas canvasx $x]
	set y [$canvas canvasy $y]
	set distx [expr $x - [set $varmarkx]]
	set disty [expr $y - [set $varmarky]]
	
	# mouvi tutto ciò che ha il tag $tag
	$canvas move $tag $distx $disty
	
	# aggiorna i mark della posizione corrente della legenda
	set $varmarkx $x
	set $varmarky $y
	
	# aggiorna i valori nelle variabili del namespace
	set varcoordx [subst $tag]coordx
	set varcoordy [subst $tag]coordy
	variable $varcoordx
	variable $varcoordy
	set $varcoordx [expr [set $varcoordx] + $distx]
	set $varcoordy [expr [set $varcoordy] + $disty]
	
}

proc dragLegend {x y} {
	variable canvas
	variable legmarkx
	variable legmarky
	
	#calcola la distanza di cui si è mosso il mouse
	set x [$canvas canvasx $x]
	set y [$canvas canvasy $y]
	set distx [expr $x - $legmarkx]
	set disty [expr $y - $legmarky]
	
	# mouvi tutto ciò che ha il tag legend
	$canvas move legend $distx $disty
	
	# aggiorna i mark della posizione corrente della legenda
	set legmarkx $x
	set legmarky $y
	
	# aggiorna legx e legy
	variable legx
	variable legy
	set legx [expr $legx + $distx]
	set legy [expr $legy + $disty]
	
	# controlla se la larghezza è sufficiente
	if $pchart::adaptwidth  {adapt width}
	if $pchart::adaptheight  {adapt height}
}

proc dragLabel {conto x y} {
	variable canvas
	variable labmarkx
	variable labmarky
	
	#calcola la distanza di cui si è mosso il mouse
	set x [$canvas canvasx $x]
	set y [$canvas canvasy $y]
	set distx [expr $x - $labmarkx]
	set disty [expr $y - $labmarky]
	
	# mouvi tutto ciò che ha il tag legend
	$canvas move label$conto $distx $disty
	
	# aggiorna i mark della posizione corrente della legenda
	set labmarkx $x
	set labmarky $y
	
	# aggiorna legx e legy
	variable arr
	set arr($conto.labcoordx) [expr $arr($conto.labcoordx) + $distx]
	set arr($conto.labcoordy) [expr $arr($conto.labcoordy) + $disty]
}

proc adapt {dimension} {
	variable canvas
	
	# trova la bbox della legenda e le dimensioni del canvas
	set bbox [$canvas bbox legend]
	set dim [subst \$pchart::$dimension]
	
	# calcola la differenza della posizione della legenda rispetto a width e height del canvas
	if {$dimension eq "width"} {
		set diff [expr $dim - [lindex $bbox 2] - 10]
	} else {
		set diff [expr $dim - [lindex $bbox 3] - 10]
	}
	
	# se la differenza è negativa, la legenda è fuori quindi ridimensiona
	if {$diff < 0} {
		if {$diff < -200} {set diff -200}
		$canvas configure -$dimension [expr $dim + abs($diff)]
	}
}

proc scan {x y} {
	variable canvas
	
	# distruggi tip precedenti
	catch {destroy $canvas.tip}
	
	# trova l'ID degli overlapping items
	set trovate [$canvas find overlapping $x $y $x $y]
	
	# per ogni tag dentro ogni ID cerca se c'è qualche serie e aggiungila alla lista tips
	set moveable 0
	set tips ""
	foreach trovata $trovate {
		foreach tag [$canvas gettags $trovata] {
			if [regexp {serie(.*)} $tag all serie] {lappend tips $serie}
			if {$tag eq "legend" || $tag eq "moveable" || $tag eq "label"} {set moveable 1}
		}
	}
	
	# se ha trovato una serie, mostra l'etichetta
	if {($pchart::tip > 0) && ($tips ne "")} {tip $tips}
	# se ha trovato la legenda o altri item col tag "moveable" cambia il cursore
	if {($pchart::move > 0) && ($moveable > 0)} {$canvas configure -cursor fleur} else {$canvas configure -cursor ""}
}

proc tip {tips} {
	variable canvas
	variable arr
	
	# crea la lista con i nomi e le coordinate
	set text ""
	foreach serie $tips {
		set x [lindex $arr($serie.data) 0]
		set y [lindex $arr($serie.data) 1]
		lappend text "$arr($serie.name)\n([format %.2f $x]:[format %.2f $y])"
	}
	
	# metti l'etichetta
	place [label $canvas.tip -text [join $text \n] -anchor w -bg white -relief solid -borderwidth 1] -x [expr [x $x] + 7] -y [y $y] -anchor w
}

proc raise {x y} {
	variable canvas
	
	# trova l'ID degli overlapping items
	set trovate [$canvas find overlapping $x $y $x $y]
	
	# per ogni tag dentro ogni ID cerca i tag
	foreach trovata $trovate {
		set tags [$canvas gettags $trovata]
		if {[lsearch $tags legend] > -1} {
			$canvas raise {legend}
		} elseif {[lsearch $tags unraiseable] == -1} {
			$canvas raise $trovata
		}
	}
	
}

proc unsetVars {} {
	foreach var {width height noffset soffset woffset eoffset xmin xmax ymin ymax canvas serie legx legy legmarkx legmarky} {
		catch {unset pchart::$var}
	}
	array unset pchart::arr
}

proc px2perc {coord px} {
	if {$coord eq "x"} {
		return [expr $px / double($::pchart::width) * 100]
	} elseif {$coord eq "y"} {
		return [expr $px / double($::pchart::height) * 100]
	}
}

proc perc2px {coord perc} {
	if {$coord eq "x"} {
		return [expr $perc * $::pchart::width / 100.0]
	} elseif {$coord eq "y"} {
		return [expr $perc * $::pchart::height / 100.0]
	}
}

proc resizeVars {newwidth newheight} {
	variable arr
	variable serie
	
	# crea una lista delle variabili da scalare
	set scalablex ""
	set scalabley ""
	foreach var {legx} {
		if ![info exists $var] {continue}
		variable $var
		lappend scalablex legx
	}
	foreach var {legy} {
		if ![info exists $var] {continue}
		variable $var
		lappend scalabley legy
	}
	
	# aggiungi alla lista le variabili che controllano le coordinate di tutte le etichette
	for {set conto 1} {$conto <= $serie} {incr conto} {
		if ![info exists arr($conto.showlab)] {continue}
		lappend scalablex arr($conto.labcoordx)
		lappend scalabley arr($conto.labcoordy)
	}
	
	# trova le posizioni in percentuale
	foreach var $scalablex {
		set $var [px2perc x [subst \$$var]]
	}
	foreach var $scalabley {
		set $var [px2perc y [subst \$$var]]
	}
	
	# aggiorna width, height e gli offset in pxls
	variable width $newwidth
	variable height $newheight
	variable noffset [perc2px y $pchart::noffsetperc]
	variable soffset [perc2px y $pchart::soffsetperc]
	variable woffset [perc2px x $pchart::woffsetperc]
	variable eoffset [perc2px x $pchart::eoffsetperc]
	
	# trova le posizioni in px date le nuove dimensioni
	foreach var $scalablex {
		set $var [perc2px x [subst \$$var]]
	}
	foreach var $scalabley {
		set $var [perc2px y [subst \$$var]]
	}
}

proc insertLine {{pendenza 1} {intersezione 0} args} {
	lappend ::pchart::history [concat insertLine $pendenza $intersezione $args]
	variable canvas
	variable xmin
	variable xmax
	variable ymin
	variable ymax
	
	# se i valori sono fuori non disegnare niente
	if {($intersezione < $xmin) && ([expr $xmax * $pendenza + $intersezione] < $ymin)} {
		return 0
	} elseif {($intersezione > $xmax) && ([expr $xmax * $pendenza + $intersezione] > $ymax)} {
		return 0
	}
	
	# stabilisci x1 e y1
	if {[expr $pendenza * $xmin + $intersezione] < $ymin} {
		# se la retta entra intersecando l'asse delle x
		set x1 [expr ($ymin - $intersezione) / $pendenza]
		set y1 $ymin
	} else {
		# se la retta entra intersecando l'asse delle y
		set x1 $xmin
		set y1 [expr $pendenza * $xmin + $intersezione]
	}
	
	# stabilisci x2 e y2
	if {[expr $xmax * $pendenza + $intersezione] > $ymax} {
		# se la retta esce toccando il massimo delle y
		set x2 [expr ($ymax - $intersezione) / $pendenza]
		set y2 $ymax
	} else {
		# se la retta esce toccando il massimo delle x
		set x2 $xmax
		set y2 [expr $xmax * $pendenza + $intersezione]
	}
	
	eval [concat $canvas create line [x $x1] [y $y1] [x $x2] [y $y2] $args]
	
	return 1
}

proc insert {type data args} {
	lappend ::pchart::history "insert $type [list $data] $args"
	variable canvas
	foreach {x y} $data {
		lappend coord [x $x] [y $y]
	}
	eval $canvas create $type $coord $args
}

proc resize {newwidth newheight} {
	# ridimensiona le variabili
	resizeVars $newwidth $newheight
	variable width $newwidth
	variable height $newheight
	
	# cancella l'array con tutti i dati (non è necessario, verrebbero comunque riscritti)
	#array unset pchart::arr
	
	# cancella il canvas e lo ridimensiona
	variable canvas
	$canvas delete all
	$canvas configure -width $width -height $height
	
	# riesegue tutti i comandi nella lista history
	foreach command $pchart::history {
		eval $command
	}
}

}
