# procedure utili in TCL/TK
# Paolo Mairano 2008

namespace eval ptk {

proc selectfont {w variabile} {
	variable cambia
	variable fontvar
	set cambia 0
	
	upvar $variabile var
	if {$var eq ""} {set var [font actual TkDefaultFont]}
	set fontvar $var
	
	foreach {a b} $var {
		variable [string range $a 1 end] $b
	}
	
	variable fontfamily $family
	variable fontsize [expr abs($size)]
	variable fontweight
	if {$weight eq "bold"} {set fontweight 1} else {set fontweight 0}
	variable fontslant
	if {$slant eq "italic"} {set fontslant 1} else {set fontslant 0}
	variable fontunderline $underline
	variable fontoverstrike $overstrike
	
	toplevel $w
	wm resizable $w 0 0
	grid [::superscrolled listbox $w.frame.lb y -height 10 -width 17] -column 0 -rowspan 2
	bind $w.frame.lb <ButtonRelease> "set ptk::fontfamily \[$w.frame.lb get \[$w.frame.lb curselection\]\]; set ptk::fontvar \[ptk::tksf_es $w\]"
	grid [ttk::labelframe $w.l -text Size:] -row 0 -column 1 -padx 20
	grid [spinbox $w.l.sp -from 4 -to 40 -width 2 -textvariable ptk::fontsize -command "set ptk::fontvar \[ptk::tksf_es $w\]"] -row 1 -column 1 -padx 20
	grid [ttk::labelframe $w.l2 -text Effects:] -row 1 -column 1 -padx 20
	pack [ttk::checkbutton $w.l2.bold -text bold -variable ptk::fontweight -command "set ptk::fontvar \[ptk::tksf_es $w\]"] -padx 10 -anchor w
	pack [ttk::checkbutton $w.l2.italic -text italic -variable ptk::fontslant -command "set ptk::fontvar \[ptk::tksf_es $w\]"] -padx 10 -anchor w
	pack [ttk::checkbutton $w.l2.underline -text underline -variable ptk::fontunderline -command "set ptk::fontvar \[ptk::tksf_es $w\]"] -padx 10 -anchor w
	pack [ttk::checkbutton $w.l2.overstrike -text overstrike -variable ptk::fontoverstrike -command "set ptk::fontvar \[ptk::tksf_es $w\]"] -padx 10 -anchor w
	grid [label $w.es -text "This is a sample." -font $var -relief sunken -bg white] -row 6 -columnspan 2 -pady 20
	grid [ttk::button $w.cancel -text Cancel -command "destroy $w"] [ttk::button $w.ok -text Ok -command "set ptk::cambia 1; destroy $w"]
	
	variable indice 0
	variable trovato 0
	foreach font [lsort -dictionary [font families]] {
		$w.frame.lb insert end $font
		if {$font eq $family} {set trovato 1}
		if {$trovato == 0} {incr indice}
	}
	$w.frame.lb selection set $indice
	$w.frame.lb see $indice
	
	tkwait window $w
	if {$cambia == 1} {set var $ptk::fontvar}
	return $var
}

proc tksf_es {w} {
	if {$ptk::fontweight == 1} {set ptk::weight bold} else {set ptk::weight normal}
	if {$ptk::fontslant == 1} {set ptk::slant italic} else {set ptk::slant roman}
	set var "-family \"$ptk::fontfamily\" -size $ptk::fontsize -weight $ptk::weight -slant $ptk::slant -underline $ptk::fontunderline -overstrike $ptk::fontoverstrike"
	$w.es configure -font $var
	return $var
}

#crea un menubutton che salva come immagine un widget (tipicamente un canvas)
proc btnSaveAsImg {btn w args} {
	
	#crea il menubutton e configuralo con le opzioni
	ttk::menubutton $btn -menu $btn.menu
	eval "$btn configure $args"
	
	#crea il menu
	menu $btn.menu -tearoff 0
	$btn.menu add command -label Postscript -command "ptk::canvas2ps $w"
	if ![catch {package require Img}] {
		$btn.menu add command -label GIF -command "ptk::w2img $w gif"
		$btn.menu add command -label JPEG -command "ptk::w2img $w jpeg"
		$btn.menu add command -label PNG -command "ptk::w2img $w png"
		$btn.menu add command -label BMP -command "ptk::w2img $w bmp"
		$btn.menu add command -label TIFF -command "ptk::w2img $w tiff"
		$btn.menu add command -label XPM -command "ptk::w2img $w xpm"
	}
	return $btn
}

#salva un widget in formato ps
proc canvas2ps {w} {
	variable canvas2ps_file [tk_getSaveFile -defaultextension .ps]
	if {$canvas2ps_file eq ""} {return}
	if [catch {focus [winfo parent $w]}] {focus $w}
	update idletasks
	set x [lindex [$w cget -scrollregion] 2]
	set y [lindex [$w cget -scrollregion] 3]
	if ![catch {$w postscript -file $canvas2ps_file -height $y -width $x}] {
		tk_messageBox -parent $w -message $canvas2ps_file
	}
}

#salva un widget in formato immagine (ext)
proc w2img {w {ext jpeg}} {
	variable w2img_file [tk_getSaveFile -defaultextension .$ext -parent $w]
	if {$w2img_file eq ""} {return}
	if [catch {set parent [winfo parent $w]}] {set parent $w}
	catch {focus $parent}
	$parent configure -cursor watch
	after 100
	update
	variable w2img_img [image create photo -format window -data $w]
	if ![catch {$w2img_img write $w2img_file -format $ext} error] {
		tk_messageBox -parent $w -message $w2img_file
	} else {bgerror $error}
	$parent configure -cursor arrow
}

#salva il testo di un text in un file txt
proc btnSaveAsTxt {btn w args} {
	ttk::button $btn
	eval "$btn configure $args"
	$btn configure -command "ptk::text2txt $w"
	return $btn
}

proc text2txt {w} {
	variable text2txt_testo [$w get 1.0 end]
	set text2txt_file [tk_getSaveFile -defaultextension .txt -parent $w]
	if {$text2txt_file eq ""} {return}
	if ![catch {open $text2txt_file w} text2txt_f] {
		puts $text2txt_f $text2txt_testo
		close $text2txt_f
		tk_messageBox -parent $w -message $text2txt_file
	}
}

#crea un toplevel che chiede di inserire una stringa
proc getString {top {msg ""} {defaultstring ""} {errormsg "Cannot proceed: invalid characters."} {extra_condition 1}} {
	variable getString_string $defaultstring
	variable getString_ok 0
	if [catch {toplevel $top}] {focus $top; return}
	grab $top
	wm resizable $top 0 0
	wm title $top ""
	pack [message $top.m -text $msg] -padx 5
	pack [ttk::entry $top.e -textvariable ptk::getString_string] -pady 10 -padx 5
	focus $top.e
	bind $top.e <Return> "$top.ok invoke"
	pack [ttk::button $top.ok -text Ok -command "
		if {\[checkstring \$ptk::getString_string\] && $extra_condition} {
			destroy $top
			set ptk::getString_ok 1
		} else {
			tk_messageBox -parent $top -message \[concat $errormsg\]
		}
	"] -padx 5
	tkwait window $top
	if {$ptk::getString_ok == 1} {return $getString_string} else {return ""}
}

proc colChange {col {parent .}} {
	variable colChange_colore [tk_chooseColor -parent $parent]
	if {$colChange_colore ne ""} {return $colChange_colore} else {return $col}
}

proc getFloat {top min max def {msg ""}} {
	if [catch {toplevel $top}] {focus $top; return}
	grab $top
	wm resizable $top 0 0
	wm title $top ""
	pack [message $top.m -text $msg] -padx 5
	pack [ttk::scale $top.scale -variable ptk::getFloat_num -from $min -to $max] -padx 5 -pady 10
	set ptk::getFloat_num $def
	pack [ttk::button $top.ok -text Ok -command "destroy $top"] -padx 5
	tkwait window $top
	return $ptk::getFloat_num
}

proc labelscale {w {text ""} {step 1} args} {
	# valori di default
	set variable ptk::labelscale
	
	# leggi gli argomenti, in modo che se è presente variable sostituisce il valore di default
	foreach {a b} $args {
		set [string range $a 1 end] $b
	}
	
	# crea il frame
	ttk::frame $w
	ttk::label $w.l1 -text $text -anchor e
	ttk::label $w.l2 -textvariable $variable -anchor w
	grid $w.l1 $w.l2 -sticky ew -padx 0 -ipadx 0
	grid [ttk::scale $w.scale -variable $variable -command "set $variable \[approssima \$$variable $step\]; set ptk::trash"] -columnspan 2
	
	# configura la scale con ogni argomento
	foreach {a b} $args {
		if {$a eq "-command"} {
			set b [concat "set $variable \[approssima \$$variable $step\]; $b"]
		}
		$w.scale configure $a $b
	}
	
	return $w
}

proc scrolledframe {frame_in {coord xy}} {
	set canvas [string range $frame_in 0 [expr [string last . $frame_in] - 1]]
	set frame_out [string range $canvas 0 [expr [string last . $canvas] - 1]]
	superscrolled canvas $canvas $coord -highlightthickness 0
	ttk::frame $frame_in
	$canvas create window 10 0 -anchor nw -window $frame_in
	bind $frame_in <Configure> "
		$canvas configure -scrollregion \"0 0 \[expr 10 + \[winfo reqwidth $frame_in\]\] \[winfo reqheight $frame_in\]\"
	"
	return $frame_out
}

proc toggleframe {b frame row {column 0}} {
	ttk::button $b -image gif_piu -command "
		if {\[$b cget -image\] eq \"gif_piu\"} {
			grid $frame -row $row -column $column
			$b configure -image gif_meno
		} else {
			grid forget $frame
			$b configure -image gif_piu
		}
	"
	return $b
}

proc imagemenu {w var images args} {
	#images deve essere una lista {immagine immagine ...}, args sono gli argomenti di ttk::menubutton (escluso textvariable?)
	ttk::menubutton $w -menu $w.menu
	catch {eval $w config $args}
	uplevel #0 [list trace add variable $var write "ptk::imagemenu_change $w"]
	menu $w.menu -tearoff 0
	foreach img $images {
		$w.menu add command -image $img -command "set $var $img"
	}
	catch {
		upvar #0 $var image
		$w configure -image $image
	}
	return $w
}

proc imagemenu_change {w varname index opt} {
	upvar #0 $varname var
	$w configure -image $var
}

proc textWithSearch {top {contenuto ""} args} {
	toplevel $top
	
	pack [set frm [ttk::frame $top.frm]] -anchor n
	ttk::label $frm.lab -text Search:
	entry $frm.entry -textvariable ptk::textwithsearch_pattern -width 10
	bind $frm.entry <KeyRelease> "ptk::searchtext $top.txtfrm.text \$ptk::textwithsearch_pattern"
	grid $frm.lab $frm.entry -padx 10
	
	pack [superscrolled text $top.txtfrm.text y] -fill both -expand yes
	if {$contenuto ne ""} {
		$top.txtfrm.text insert 1.0 $contenuto
	}
	eval $top.txtfrm.text config $args
	$top.txtfrm.text tag configure sele -background orange
	return $top.txtfrm.text
}

proc searchtext {w pattern} {
	$w tag remove sele 1.0 end
	if {$pattern eq ""} {return}
	set start 1.0
	set volte 0
	while {$start ne ""} {
		incr volte
		set start [$w search -nocase -count count $pattern "$start +1 chars" end]
		catch {
			if {$volte == 1} {$w see $start}
			$w tag add sele $start "$start +$count chars"
		}
	}
}









}

# crea un widget scrollato, es. 'superscrolled text .frame.text xy -width 10 -height 10'
proc superscrolled {widget name coord args} {
	set frame_name [string range $name 0 [expr [string last . $name] - 1]]
	ttk::frame $frame_name
	set scrolly [join [list $name "scrolly"] "_"]
	set scrollx [join [list $name "scrollx"] "_"]
	grid [$widget $name] -row 1 -column 1 -sticky news
	if {[string match "*y*" $coord]} {
		grid [ttk::scrollbar $scrolly -orient vertical -command "$name yview"] -row 1 -column 2 -sticky ns
		$name configure -yscrollcommand "$scrolly set"
	}
	if {[string match "*x*" $coord]} {
		grid [ttk::scrollbar $scrollx -orient horizontal -command "$name xview"] -row 2 -column 1 -sticky ew
		$name configure -xscrollcommand "$scrollx set"
	}
	grid rowconfigure $frame_name 1 -weight 1
	grid columnconfigure $frame_name 1 -weight 1
	eval "$name configure $args"
	return $frame_name
}

# abbina un tooltip con testo $testo al widget $nome
if [catch {package require tooltip}] {
	proc suggerimento {nome testo} {
		toplevel $nome.suggerimento -class Tooltip
		pack [message $nome.suggerimento.label -text $testo -bg lightyellow -relief solid -borderwidth 1]
		wm overrideredirect $nome.suggerimento 1
		wm transient $nome.suggerimento $nome
		#wm attributes $nome.suggerimento -topmost 1
		wm positionfrom $nome.suggerimento program
		wm withdraw $nome.suggerimento
		bind $nome <Motion> "
			wm geometry $nome.suggerimento +\[expr 10 + \[winfo pointerx $nome\]\]+\[expr 20 + \[winfo pointery $nome\]\]; wm deiconify $nome.suggerimento; raise $nome.suggerimento
		"
		bind $nome <Leave> "wm withdraw $nome.suggerimento"
	}
} else {
	proc suggerimento {nome testo} {
		tooltip::tooltip $nome $testo
	}
}
