#----- procedure generali per la gestione del report -----

proc rreport {tier {reportpath ""}} {
	if {$reportpath eq ""} {set reportpath $::reportpath}
	global l
	set report [file join ~ .correlatore report $reportpath]
	if {![file exists $report]} {return ""}
	if {[catch {open $report r} f]} {
		cannot read report
		return ""
	}
	set agisci 0
	set risultato ""
	while {[gets $f riga] >= 0} {
		set riga [split $riga \t]
		if {[string match FILE* [lindex $riga 0]] & [string match $tier [lindex $riga 1]]} {set agisci 1}
		if {[string match FILE* [lindex $riga 0]] & ![string match $tier [lindex $riga 1]]} {set agisci 0}
		if {$agisci eq 1} {lappend risultato [join $riga \t]}
	}
	close $f
	return $risultato
}

proc wreport {data {reportpath ""}} {
	if {$reportpath eq ""} {set reportpath $::reportpath}
	global l
	set report [file join ~ .correlatore report $reportpath]
	if {[catch {open $report a} f]} {
		cannot write report
		return ""
	}
	puts $f $data
	close $f
	return ok
}

proc mreport {tier key value {reportpath ""}} {
	if {$reportpath eq ""} {set reportpath $::reportpath}
	set scritto 0
	set tier_corrente ""
	set f [open [file join ~ .correlatore report $reportpath] r]
	set file_vecchio [read $f]
	close $f
	set f [open [file join ~ .correlatore report $reportpath] w]
	foreach line [split $file_vecchio \n] {
		set a [lindex $line 0]
		if {$a eq "FILE"} {
			puts $f $line; set tier_corrente [lindex $line 1]
		} elseif {$a eq $key} {
			if {$tier_corrente eq $tier} {puts $f "$key\t$value"; set scritto 1} {puts $f $line}
		} elseif {$a eq ""} {
			continue
		} elseif {$a eq "--"} {
			if {$tier_corrente eq $tier && $scritto eq 0} {puts $f "$key\t$value\n$line"} {puts $f $line}
		} else {puts $f $line}
	}
	close $f
}

proc report_tiers {{reportpath ""}} {
	if {$reportpath eq ""} {set reportpath $::reportpath}
	set tiers ""
	set report [file join ~ .correlatore report $reportpath]
	if {![file exists $report]} {return ""}
	if {[catch {open $report r} f]} {
		cannot read report
		return ""
	}
	while {[gets $f riga] >= 0} {
		if {[string match FILE* $riga]} {
			set riga [split $riga \t]
			lappend tiers [lindex $riga 1]
		}
	}
	close $f
	return $tiers
}

proc report_names {} {
	foreach f [glob -nocomplain [file join ~ .correlatore report *.txt]] {
		lappend txts [file tail $f]
	}
	if ![info exists txts] {set txts ""}
	return [lsort -dictionary $txts]
}

proc lista_report {name {height 8} {width 15}} {
	# crea un frame con la lista di tutti i report presenti
	global l reportpath reptxts
	set reptxts ""
	set conto 0
	if [catch {set txts [glob [file join ~ .correlatore report *.txt]]}] {set txts standard.txt}
	if {![string match *standard.txt* $txts]} {lappend txts standard.txt}
	foreach txt [lsort -dictionary $txts] {
		lappend reptxts [file tail $txt]
	}
	catch {destroy $name}
	superscrolled listbox $name.listbox xy -width $width -height $height -listvariable reptxts
	bind $name.listbox <<ListboxSelect>> {
		set reportpath [%W get [%W curselection]]
		wconfigure reportpath $reportpath
	}
	select_reportpath $name.listbox
	return $name
}

proc select_reportpath {lb} {
	set index [lsearch $::reptxts $::reportpath]
	if {$index < 0} {set index 0}
	$lb selection set $index
	$lb see $index
}

proc report_problem {} {
	array set mess {
		it "Attenzione: qualche altra finestra di correlatore sta accendendo ai report. È necessario chiuderla prima di proseguire."
		en "Warning: some other window of Correlatore is accessing the reports. You need to close it before continuing."
	}
	tk_messageBox -message $mess($::l)
}


#----- procedure per gestire la finestra di modifica di un report -----


proc apri_report {{reportpath ""}} {
	if {$reportpath eq ""} {set reportpath $::reportpath}
	global l
	
	#----- controlla che non sia già aperta la finestra di report e creala
	if [winfo exists .report] {
		array set mess {
			it "È già aperta una finestra con questo report. Non è possibile aprirne 2 contemporaneamente."
			en "One report is already open in some other window. It is not possible to edit 2 reports at the same time."
		}
		tk_messageBox -message $mess($l)
		return
	}
	
	toplevel .report
	wm resizable .report 0 0
	wm title .report "Report: [string range $reportpath 0 end-4]"
	wm protocol .report WM_DELETE_WINDOW report_chiedi_salva
	.charts.apri_report config -state disabled
	
	#----- crea l'array arr; i names sono i nomi degli item, i values sono delle liste che contengono i dati del report.
	global arr
	array unset arr
	foreach tier [report_tiers] {
		set arr($tier) [concat [rreport $tier] \n]
	}
	
	#----- crea i bottoni sopra
	grid [ttk::frame .report.top] -row 0 -column 0 -columnspan 6 -sticky news
	array set mess {it "Nuovo item" en "New item"}
	ttk::button .report.top.nuovo -text $mess($l) -image gif_piu -compound top -command {report_nuovo_tier}
	array set mess {it "Elimina item" en "Delete item"}
	ttk::button .report.top.elimina -text $mess($l) -image gif_meno -state disabled -compound top -command {report_elimina_tier}
	array set mess {it "Rinomina item" en "Rename item"}
	ttk::button .report.top.rinomina -text $mess($l) -image gif_cartella -state disabled -compound top -command {report_rinomina_tier}
	array set mess {it "Calcola media" en "Calculate mean"}
	ttk::button .report.top.media -text $mess($l) -image gif_formule -state disabled -compound top -command {report_media_tier}
	array set mess {it "Salva" en "Save"}
	ttk::button .report.top.salva -text $mess($l) -image gif_salva -state disabled -compound top -command "salva_report $reportpath"
	array set mess {it "Chiudi" en "Close"}
	ttk::button .report.top.chiudi -text $mess($l) -image gif_esci -compound top -command {report_chiedi_salva; return}
	grid .report.top.nuovo .report.top.elimina .report.top.rinomina .report.top.media .report.top.salva .report.top.chiudi -sticky news -row 0
	grid columnconfigure .report.top "0 1 2 3 4 5" -weight 1 -uniform group1
	
	#-----crea le etichette
	set labcount 0
	foreach {it en} {Items Items Correlati Metrics "Valori A" "Values A" "Valori B" "Values B" "ErrBar A" "ErrBar A" "ErrBar B" "ErrBar B"} {
		grid [label .report.l$labcount -text [subst $$l] -relief raised] -row 1 -column $labcount -sticky ew
		incr labcount
	}
	
	#----- crea la listbox con i nomi degli item
	grid [superscrolled listbox .report.sx.tiers xy -width 12 -height 20 -selectmode extended] -row 2 -column 0 -sticky we
	foreach tier [lsort -dictionary [array names arr]] {
		.report.sx.tiers insert end $tier
	}
	
	#----- crea le 4 listbox con i dati del report
	grid [listbox .report.centro -width 8 -height 21 -relief flat -listvariable contenuto0] -row 2 -column 1 -sticky we
	global contenuto0
	set contenuto0 ""
	grid [listbox .report.ds -width 19 -height 21 -relief flat -listvariable contenuto1] -row 2 -column 2 -sticky we
	global contenuto1
	set contenuto1 ""
	grid [listbox .report.ds2 -width 19 -height 21 -relief flat -listvariable contenuto2] -row 2 -column 3 -sticky we
	global contenuto2
	set contenuto2 ""
	grid [listbox .report.ds3 -width 7 -height 21 -relief flat -listvariable contenuto3] -row 2 -column 4 -sticky we
	global contenuto3
	set contenuto3 ""
	grid [listbox .report.ds4 -width 7 -height 21 -relief flat -listvariable contenuto4] -row 2 -column 5 -sticky we
	global contenuto4
	set contenuto4 ""
	
	#----- bindings al clic del tasto sinistro su un item
	foreach lb {.report.sx.tiers .report.centro .report.ds .report.ds2 .report.ds3 .report.ds4} {
		bind $lb <<ListboxSelect>> report_abdisab
	}
	.report.sx.tiers selection set 0
	report_abdisab
}

proc report_abdisab {} {
	global contenuto0 contenuto1 contenuto2 contenuto3 contenuto4
	global arr
	#----- se almeno un item è selezionato abilita il pulsante per eliminare
	if {[.report.sx.tiers curselection] >= 0} {
		.report.top.elimina configure -state normal
		# se è selezionato solo 1 item abilita il pulsante per rinominare
		if {[llength [.report.sx.tiers curselection]] == 1} {
			.report.top.rinomina configure -state normal
		} else {
			.report.top.rinomina configure -state disabled
		}
		# se sono selezionati almeno 2 item abilita il pulsante per le medie
		if {[llength [.report.sx.tiers curselection]] > 1} {
			.report.top.media configure -state normal
		} else {
			.report.top.media configure -state disabled
		}
		
		#----- visualizza nelle lisbox di destra i dati dell'item
		set contenuto0 ""
		set contenuto1 ""
		set contenuto2 ""
		set contenuto3 ""
		set contenuto4 ""
		catch {
			set tier [.report.sx.tiers get [.report.sx.tiers curselection]]
			foreach item $arr($tier) {
				set item [split $item \t]
				lappend contenuto0 [lindex $item 0]
				lappend contenuto1 [lindex $item 1]
				lappend contenuto2 [lindex $item 2]
				lappend contenuto3 [lindex $item 3]
				lappend contenuto4 [lindex $item 4]
			}
		}
	} else {
		.report.top.elimina configure -state disabled
		.report.top.rinomina configure -state disabled
		.report.top.media configure -state disabled
	}
}

proc salva_report {{reportpath ""}} {
	if {$reportpath eq ""} {set reportpath $::reportpath}
	global l
	global arr
	#cancella il file vecchio
	file delete [file join ~ .correlatore report $reportpath]
	#crea il file
	set f [open [file join ~ .correlatore report $reportpath] w]
	close $f
	#scrivici i tier
	foreach tier [array names arr] {
		wreport [join $arr($tier) \n]
	}
	.report.top.salva configure -state disabled
}

proc report_getname {{default_name ""}} {
	global l
	array set mess {it "Inserire nuovo nome:" en "Insert new name:"}
	array set errormess {
		it "Il nome contiene caratteri non permessi oppure è già presente nel report." 
		en "The name you entered either contains invalid characters or is already present in the report."
	}
	ptk::getString .report.new_name $mess($l) $default_name $errormess($l) {[lsearch [array names arr] $ptk::getString_string] == -1}
}

proc report_elimina_tier {} {
	global l
	global arr
	
	#----- controlla che almeno un item sia selezionato, se no avverti e ritorna
	if [catch {set selection [.report.sx.tiers curselection]}] {
		array set mess {it "È necessario selezionare un tier." en "You need to select a tier."}
		tk_messageBox -parent .report -message $mess($l)
		return
	}
	
	#----- cancella ogni item selezionato
	foreach item $selection {
		array unset arr [.report.sx.tiers get $item]
	}
	
	#----- svuota le listbox e ririempi la prima con gli item rimasti
	foreach lb {.report.sx.tiers .report.centro .report.ds .report.ds2 .report.ds3 .report.ds4} {
		$lb delete 0 end
	}
	foreach tier [lsort -dictionary [array names arr]] {
		.report.sx.tiers insert end $tier
	}
	
	#----- (dis)abilita i buttons
	report_abdisab
	.report.top.salva configure -state normal
}

proc report_rinomina_tier {} {
	global l
	global arr
	
	#----- controlla che ci sia uno e un solo item selezionato, se no avverti e ritorna
	if {[catch {set tier [.report.sx.tiers get [.report.sx.tiers curselection]]}]} {
		array set mess {it "È necessario selezionare uno e un solo tier." en "You need to select one and only one tier."}
		tk_messageBox -parent .report -message $mess($l)
		return
	}
	
	#disabilita tasto rinomina
	.report.top.rinomina configure -state disabled
	
	#----- chiedi nuovo nome; se esiste già un item con lo stesso nome ripeti il ciclo, se no procedi
	if {[set nuovo_nome [report_getname $tier]] eq ""} {report_abdisab; return}
	
	#----- cambia il nome nei dati dell'array
	lset arr($tier) 0 "FILE\t$nuovo_nome"
	#----- crea un nuovo item con il nuovo nome
	set arr($nuovo_nome) $arr($tier)
	#----- elimina il vecchio item con il vecchio nome
	array unset arr $tier
	
	#----- svuota le listbox e ririempi la prima con gli item rimasti
	foreach lb {.report.sx.tiers .report.centro .report.ds .report.ds2 .report.ds3 .report.ds4} {
		$lb delete 0 end
	}
	foreach tier [lsort -dictionary [array names arr]] {
		.report.sx.tiers insert end $tier
	}
	
	#----- (dis)abilita i buttons
	report_abdisab
	.report.top.salva configure -state normal
}

proc report_media_tier {} {
	global l
	global arr
	
	#----- controlla che ci siano almeno 2 item selezionati, se no avverti e ritorna
	set selection [.report.sx.tiers curselection]
	if {[llength $selection] < 2} {
		array set mess {it "È necessario selezionare almeno 2 item." en "You need to select at least 2 items."}
		tk_messageBox -parent .report -message $mess($l)
		return
	}
	
	#disabilita tasto media
	.report.top.media configure -state disabled
	
	#----- ottieni nome del nuovo item
	if {[set nuovo_nome [report_getname]] eq ""} {report_abdisab; return}
	
	#----- decidi stdev o stderr
	array set mess {
		it "INFORMAZIONE IMPORTANTE\nIl calcolo della media permette anche di creare delle barre a forma di + sul grafico.\nFino alla versione 2.1 (inclusa), le barre mostravano la deviazione standard dei valori di cui si calcola la media. A partire dalla versione 2.2 le barre possono mostrare l'errore standard (come nei famosi grafici pubblicati da Ramus et al, 1999). Se il tuo report contiene dati calcolati con una versione precedente alla 2.1, e' consigliabile continuare a calcolare la deviazione standard per omogeneita' dei dati.\nPremi YES per avere l'errore standard, o NO per avere la deviazione standard."
		en "IMPORTANT INFORMATION\nWhen calculating the mean, Correlatore also calculates values for + bars on your chart.\nUp to (and including) version 2.1, the bars showed the standard deviation of the data. Starting from version 2.2 the bars can show the standard error, if you prefer (like in the famous charts published by Ramus et al., 1999). However, if your report already contains data calculated with previous versions of Correlatore, it is advisable to continue using the standard deviation for homogeneity.\nPress YES to get the standard error, or NO to get the standard deviation."
	}
	if [tk_messageBox -message $mess($l) -type yesno -parent .report] {
		set chosenBarType stderr
	} else {
		set chosenBarType stdev
	}
	
	#----- crea lista itemmedianti con i nomi degli item su cui calcolare la media
	foreach index $selection {
		lappend itemmedianti [.report.sx.tiers get $index]
	}
	
	#----- crea le liste intV intC pause etc. che contengono i valori corrispondenti di ognuno degli item su cui calcolare la media
	foreach item $itemmedianti {
		lappend intV [lindex $arr($item) 1 1]
		lappend intC [lindex $arr($item) 2 1]
		lappend pause [lindex $arr($item) 3 1]
		lappend Vmean [lindex $arr($item) 4 1]
		lappend Cmean [lindex $arr($item) 5 1]
		lappend Vperc [lindex $arr($item) 6 1]
		lappend Vdev [lindex $arr($item) 7 1]
		lappend Vdev_medie [lindex $arr($item) 7 2]
		lappend Cdev [lindex $arr($item) 8 1]
		lappend Cdev_medie [lindex $arr($item) 8 2]
		lappend varcoV [lindex $arr($item) 9 1]
		lappend varcoV_medie [lindex $arr($item) 9 2]
		lappend varcoC [lindex $arr($item) 10 1]
		lappend varcoC_medie [lindex $arr($item) 10 2]
		lappend Vrpvi [lindex $arr($item) 11 1]
		lappend Vrpvi_medie [lindex $arr($item) 11 2]
		lappend Crpvi [lindex $arr($item) 12 1]
		lappend Crpvi_medie [lindex $arr($item) 12 2]
		lappend Vnpvi [lindex $arr($item) 13 1]
		lappend Vnpvi_medie [lindex $arr($item) 13 2]
		lappend Cnpvi [lindex $arr($item) 14 1]
		lappend Cnpvi_medie [lindex $arr($item) 14 2]
		lappend Vcci [lindex $arr($item) 15 1]
		lappend Vcci_medie [lindex $arr($item) 15 2]
		lappend Ccci [lindex $arr($item) 16 1]
		lappend Ccci_medie [lindex $arr($item) 16 2]
	}
	
	#----- crea la lista risultato con il nuovo nome, le somme di intV intC e pause, e le medie di ogni correlato
	lappend arr($nuovo_nome) "FILE\t$nuovo_nome"
	lappend arr($nuovo_nome) "intV\t[sum $intV]"
	lappend arr($nuovo_nome) "intC\t[sum $intC]"
	lappend arr($nuovo_nome) "pause\t[sum $pause]"
	lappend arr($nuovo_nome) "Vmean\t[mean $Vmean]"
	lappend arr($nuovo_nome) "Cmean\t[mean $Cmean]"
	lappend arr($nuovo_nome) "Vperc\t[mean $Vperc]\t0\t[format %.2f [$chosenBarType $Vperc]]\t0"
	lappend arr($nuovo_nome) "Vdev\t[mean $Vdev]\t[mean $Vdev_medie]\t[format %.2f [$chosenBarType $Vdev]]\t[format %.2f [$chosenBarType $Vdev_medie]]"
	lappend arr($nuovo_nome) "Cdev\t[mean $Cdev]\t[mean $Cdev_medie]\t[format %.2f [$chosenBarType $Cdev]]\t[format %.2f [$chosenBarType $Cdev_medie]]"
	lappend arr($nuovo_nome) "varcoV\t[mean $varcoV]\t[mean $varcoV_medie]\t[format %.2f [$chosenBarType $varcoV]]\t[format %.2f [$chosenBarType $varcoV_medie]]"
	lappend arr($nuovo_nome) "varcoC\t[mean $varcoC]\t[mean $varcoC_medie]\t[format %.2f [$chosenBarType $varcoC]]\t[format %.2f [$chosenBarType $varcoC_medie]]"
	lappend arr($nuovo_nome) "Vrpvi\t[mean $Vrpvi]\t[mean $Vrpvi_medie]\t[format %.2f [$chosenBarType $Vrpvi]]\t[format %.2f [$chosenBarType $Vrpvi_medie]]"
	lappend arr($nuovo_nome) "Crpvi\t[mean $Crpvi]\t[mean $Crpvi_medie]\t[format %.2f [$chosenBarType $Crpvi]]\t[format %.2f [$chosenBarType $Crpvi_medie]]"
	lappend arr($nuovo_nome) "Vnpvi\t[mean $Vnpvi]\t[mean $Vnpvi_medie]\t[format %.2f [$chosenBarType $Vnpvi]]\t[format %.2f [$chosenBarType $Vnpvi_medie]]"
	lappend arr($nuovo_nome) "Cnpvi\t[mean $Cnpvi]\t[mean $Cnpvi_medie]\t[format %.2f [$chosenBarType $Cnpvi]]\t[format %.2f [$chosenBarType $Cnpvi_medie]]"
	lappend arr($nuovo_nome) "Vcci\t[mean $Vcci]\t[mean $Vcci_medie]\t[format %.2f [$chosenBarType $Vcci]]\t[format %.2f [$chosenBarType $Vcci_medie]]"
	lappend arr($nuovo_nome) "Ccci\t[mean $Ccci]\t[mean $Ccci_medie]\t[format %.2f [$chosenBarType $Ccci]]\t[format %.2f [$chosenBarType $Ccci_medie]]"
	lappend arr($nuovo_nome) "colour\t[shuffle]"
	lappend arr($nuovo_nome) "border\tblack"
	lappend arr($nuovo_nome) "symbol\tc"
	lappend arr($nuovo_nome) "--\t"
	
	#----- svuota le listbox e ririempi la prima con gli item rimasti
	foreach lb {.report.sx.tiers .report.centro .report.ds .report.ds2 .report.ds3 .report.ds4} {
		$lb delete 0 end
	}
	foreach tier [lsort -dictionary [array names arr]] {
		.report.sx.tiers insert end $tier
	}
	
	#----- (dis)abilita i buttons
	report_abdisab
	.report.top.salva configure -state normal
}

proc report_nuovo_tier {} {
	global l
	global arr
	global nuovo_item
	global nuovo_item2
	global nuovo_item3
	global nuovo_item4
	global report_nuovo_tier_ok
	set report_nuovo_tier_ok 0
	array unset nuovo_item
	array unset nuovo_item2
	array unset nuovo_item3
	array unset nuovo_item4
	
	.report.top.nuovo configure -state disabled
	
	#crea nuova finestra e l'istruzione iniziale
	if [catch {toplevel .report.newitem}] {focus .report.newitem; return}
	array set mess {it "Nuovo item" en "New item"}
	wm title .report.newitem $mess($l)
	#array set mess {it "Riempire i campi." en "Fill in the fields."}
	set conto 0
	foreach {it en} {correlati metrics "valore A" "value A" "valore B" "value B" "stdev A" "stdev A" "stdev B" "stdev B"} {
		grid [label .report.newitem.istr$conto -text [set $l] -relief raised] -row 0 -column $conto -sticky we -ipadx 10
		incr conto
	}
	
	#per ogni riga crea una label e 1 entry
	foreach riga {FILE intV intC pause Vmean Cmean Vperc Vdev Cdev varcoV varcoC Vrpvi Crpvi Vnpvi Cnpvi Vcci Ccci} {
		set nuovo_item($riga) ""
		label .report.newitem.l$riga -text $riga
		entry .report.newitem.e$riga -textvariable nuovo_item($riga) -width 15
		catch {grid .report.newitem.l$riga .report.newitem.e$riga}
	}
	
	#crea le entry 2 e 3 nelle righe in cui servono
	set rowcount 7
	foreach riga {Vperc Vdev Cdev varcoV varcoC Vrpvi Crpvi Vnpvi Cnpvi Vcci Ccci} {
		grid [entry .report.newitem.f$riga -textvariable nuovo_item2($riga) -width 15] -row $rowcount -column 2
		grid [entry .report.newitem.g$riga -textvariable nuovo_item3($riga) -width 15] -row $rowcount -column 3
		grid [entry .report.newitem.h$riga -textvariable nuovo_item4($riga) -width 15] -row $rowcount -column 4
		incr rowcount
	}
	
	#crea pulsanti annulla e conferma
	grid [ttk::frame .report.newitem.b] -columnspan 5 -pady {10 0}
	array set mess {it Annulla en Cancel}
	ttk::button .report.newitem.b.annulla -text $mess($l) -image gif_chiudi -compound left -command {set report_nuovo_tier_ok 0; destroy .report.newitem}
	ttk::button .report.newitem.b.ok -text Ok -image gif_ok -compound left -command {
		if ![checkstring $nuovo_item(FILE)] {
			#se il nome non è valido avverti e non procedere
			array set mess {it "Impossibile procedere: il nome contiene caratteri non validi." en "Cannot proceed: the name contains invalid characters"}
			tk_messageBox -parent .report.newitem -message $mess($l)
		} elseif {$nuovo_item(FILE) eq ""} {
			#se il nome è vuoto avverti e non procedere
			array set mess {it "Impossibile procedere: inserire un nome." en "Cannot proceed: insert a name."}
			tk_messageBox -parent .report.newitem -message $mess($l)
		} elseif {[lsearch [array names arr] $nuovo_item(FILE)] >= 0} {
			#se esiste già un item con lo stesso nome avverti e non procedere
			array set mess {it "Impossibile procedere: è già presente un tier con questo nome." en "Cannot proceed: there is already a tier with this name."}
			tk_messageBox -parent .report.newitem -message $mess($l)
		} else {
			#se esiste anche solo un campo che non è un numero avverti e non procedere
			set report_nuovo_tier_ok 1
			set nuovo_tier_riga ""
			foreach riga {intV intC pause Vmean Cmean Vperc Vdev Cdev varcoV varcoC Vrpvi Crpvi Vnpvi Cnpvi Vcci Ccci} {
				if ![string is double $nuovo_item($riga)] {
					set report_nuovo_tier_ok 0
					set nuovo_tier_riga $riga
					break
				}
				if {$nuovo_item($riga) eq ""} {
					set nuovo_item($riga) 0
				}
			}
			foreach riga {Vperc Vdev Cdev varcoV varcoC Vrpvi Crpvi Vnpvi Cnpvi Vcci Ccci} {
				if {![string is double $nuovo_item2($riga)] || ![string is double $nuovo_item3($riga)] || ![string is double $nuovo_item4($riga)]} {
					set report_nuovo_tier_ok 0
					set nuovo_tier_riga $riga
					break
				}
				if {$nuovo_item2($riga) eq ""} {
					set nuovo_item2($riga) 0
				}
				if {$nuovo_item3($riga) eq ""} {
					set nuovo_item3($riga) 0
				}
				if {$nuovo_item4($riga) eq ""} {
					set nuovo_item4($riga) 0
				}
			}
			
			if {$report_nuovo_tier_ok == 1} {
				destroy .report.newitem
			} else {
				array set mess {it "non è un numero valido." en "is not a valid number."}
				tk_messageBox -parent .report.newitem -message "$nuovo_tier_riga $mess($l)"
			}
		}
	}
	pack .report.newitem.b.annulla .report.newitem.b.ok -side left -fill x -expand yes -padx 20
	
	#aspetta che si prema il tasto Ok o Annulla
	tkwait window .report.newitem
	if {$report_nuovo_tier_ok != 1} {.report.top.nuovo configure -state normal; return}
	
	#crea una lista con i dati
	set futuro ""
	foreach riga {FILE intV intC pause Vmean Cmean} {
		lappend futuro "$riga\t$nuovo_item($riga)"
	}
	foreach riga {Vperc Vdev Cdev varcoV varcoC Vrpvi Crpvi Vnpvi Cnpvi Vcci Ccci} {
		lappend futuro "$riga\t$nuovo_item($riga)\t$nuovo_item2($riga)\t$nuovo_item3($riga)\t$nuovo_item4($riga)"
	}
	lappend futuro "colour\t[shuffle]" "border\tblack" "symbol\tc" "--\t"
	
	#aggiungi ad arr il nuovo item
	set item $nuovo_item(FILE)
	set arr($item) $futuro
	
	#----- svuota le listbox e ririempi la prima con gli item rimasti
	foreach lb {.report.sx.tiers .report.centro .report.ds .report.ds2 .report.ds3 .report.ds4} {
		$lb delete 0 end
	}
	foreach tier [lsort -dictionary [array names arr]] {
		.report.sx.tiers insert end $tier
	}
	
	#----- (dis)abilita i buttons
	report_abdisab
	.report.top.salva configure -state normal
	.report.top.nuovo configure -state normal
}

proc report_chiedi_salva {} {
	global l
	if {[.report.top.salva cget -state] ne "disabled"} {
		array set mess {it "Ci sono modifiche non salvate. Uscire comunque?" en "There are unsaved changes. Do you want to exit anyway?"}
		if [tk_messageBox -message $mess($l) -parent .report -type yesno] {
			destroy .report
			.charts.apri_report config -state normal
		}
	} else {
		destroy .report
		.charts.apri_report config -state normal
	}
}


#----- procedure per gestire la cartella dei report -----


proc get_report {} {
	global l reportpath nomereport reportpath_scelto
	set reportpath_scelto 0
	
	# apri la finestra di scelta del report controllando che non esista già.
	if [catch {toplevel .cambia_report}] {return $reportpath}
	wm resizable .cambia_report 0 0
	wm title .cambia_report Report
	
	array set mess {it "Selezionare il report che si desidera utilizzare." en "Select the report you would like to use."}
	grid [ttk::label .cambia_report.l1 -text "$mess($l)"] -row 0 -pady 15 -padx 10 -columnspan 2
	
	# crea la lista $txts di file nella cartella di report + standard.txt e crea la lista
	grid [lista_report .cambia_report.f] -row 1 -column 0 -sticky news -padx 10
	grid [set btns [ttk::frame .cambia_report.btns]] -row 1 -column 1 -sticky news -padx "0 10"
	array set mess {it Clona en Clone}
	pack [ttk::button $btns.clona -image gif_piu -compound left -text $mess($l) -command {clona_report .cambia_report.f.listbox}] -fill x
	array set mess {it Rinomina en Rename}
	pack [ttk::button $btns.rinomina -image gif_cartella -compound left -text $mess($l) -command {rinomina_report .cambia_report.f.listbox}] -fill x
	array set mess {it Elimina en Delete}
	pack [ttk::button $btns.elimina -image gif_meno -compound left -text $mess($l) -command {elimina_report .cambia_report.f.listbox}] -fill x
	array set mess {it Importa en Import}
	pack [ttk::button $btns.importa -image gif_importa -compound left -text $mess($l) -command {importa_report .cambia_report.f.listbox}] -fill x
	array set mess {it Esporta en Export}
	pack [ttk::button $btns.esporta -image gif_rifai -compound left -text $mess($l) -command {esporta_report .cambia_report.f.listbox}] -fill x
	
	# crea la parte bassa della finestra, per creare un nuovo report
	grid [ttk::frame .cambia_report.ff] -row 2 -pady 10 -columnspan 2 -padx 10
	array set mess {it "Nuovo report: " en "New report: "}
	pack [ttk::label .cambia_report.ff.label -text $mess($l)] -side left
	pack [ttk::entry .cambia_report.ff.entry -width 15 -textvariable nomereport] -side left
	pack [ttk::label .cambia_report.ff.label2 -text .txt] -side left
	bind .cambia_report.ff.entry <Return> {.cambia_report.ff.add invoke}
	array set mess {it Crea en Create}
	pack [ttk::button .cambia_report.ff.add -text $mess($l) -image gif_avanti -compound left -command "aggiungi_report .cambia_report.f"] -side left -padx {10 0}
	grid [ttk::button .cambia_report.chiudi -text Ok -image gif_ok -compound left -command {set reportpath_scelto 1; destroy .cambia_report}] -row 3 -columnspan 2
	focus .cambia_report.chiudi
	
	tkwait window .cambia_report
	.charts.active config -values [report_names]
	
	if {$reportpath_scelto} {
		return $reportpath
	} else {
		return ""
	}
}

proc aggiungi_report {nome} {
	global reportpath nomereport l
	
	# controlla che il nuovo nome ci sia, sia a posto e non sia già presente, se no non crea niente
	if {![info exists nomereport] || ($nomereport eq "")} {
		array set mess {it "Specificare un nuovo nome." en "Please specify a new name."}
		tk_messageBox -message $mess($l) -parent .cambia_report
		return
	}
	if ![checkstring $nomereport] {
		array set mess {it "Impossibile procedere: il nome contiene spazi o altri caratteri non utilizzabili." en "Cannot proceed: the name contains spaces or other inappropriate characters."}
		tk_messageBox -message $mess($l) -parent .cambia_report
		return
	}
	if ![catch {glob [file join ~ .correlatore report $nomereport.txt]}] {
		array set mess {it "Impossibile procedere: è già presente un report con lo stesso nome." en "Cannot proceed: a report with this name is already present."}
		tk_messageBox -message $mess($l) -parent .cambia_report
		return
	}
	
	# crea il file di report
	if [catch {set f [open [file join ~ .correlatore report $nomereport.txt] w]}] {cannot write $nomereport; return}
	close $f
	
	# cambia le variabili, il file di configurazione e aggiorna i bottoni attivi/disabilitati
	set reportpath $nomereport.txt
	wconfigure reportpath $reportpath
	
	# ricrea la lista dei report
	grid [lista_report .cambia_report.f] -row 1 -sticky news -padx 10
}

proc elimina_report {lb} {
	set file [$lb get [$lb curselection]]
	global reportpath l reptxts
	array set mess {it "Questo distruggerà tutti i dati dentro al report. Procedere?" en "This will destroy all data in the report. Do you want to proceed?"}
	if {[tk_messageBox -type yesno -message $mess($l) -parent $lb] eq "no"} {return}
	file delete -force [file join ~ .correlatore report $file]
	if {$reportpath eq $file} {
		set reportpath standard.txt
		wconfigure reportpath $reportpath
	}
	grid [lista_report [winfo parent $lb]] -row 1 -column 0 -sticky news -padx 10
}

proc rinomina_report {lb} {
	global reportpath l
	set vecchionome [$lb get [$lb curselection]]
	if [catch {set txts [glob [file join ~ .correlatore report *.txt]]}] {set txts "standard.txt"}
	
	# chiedi nuovo nome
	array set mess {it "Nuovo nome:" en "Insert new name:"}
	array set errormess {it "Il nome contiene caratteri non validi o è già presente." en "This name either contains invalid characters or is already present."}
	set nuovonome [ptk::getString .cambia_report.getname $mess($l) [string range $vecchionome 0 end-4] $errormess($l) "\[catch {glob \[file join ~ .correlatore report \$ptk::getString_string.txt\]}\]"]
	if {$nuovonome eq ""} {return}
	
	# rinomina vecchionome con nuovonome
	file_rename [file join ~ .correlatore report $vecchionome] [file join ~ .correlatore report $nuovonome.txt]
	if {$reportpath eq $vecchionome} {
		set reportpath $nuovonome.txt; wconfigure reportpath $reportpath
	}
	grid [lista_report [winfo parent $lb]] -row 1 -column 0 -sticky news -padx 10
}

proc clona_report {lb} {	
	global reportpath
	set daclonare [$lb get [$lb curselection]]
	set nomereport "[file rootname $daclonare]_clone.txt"
	
	# copia il file di report
	file_clone [file join ~ .correlatore report $daclonare] [file join ~ .correlatore report $nomereport]
	#if [catch {}] {cannot write $nomereport; return}
	
	# cambia le variabili, il file di configurazione e aggiorna i bottoni attivi/disabilitati
	set reportpath $nomereport
	wconfigure reportpath $reportpath
	
	# ricrea la lista dei report
	grid [lista_report [winfo parent $lb]] -column 0 -row 1 -sticky news -padx 10
}

proc importa_report {lb} {
	global l
	if {[set file [tk_getOpenFile]] eq ""} {return}
	set nome [file tail $file]
	
	# se esiste già un report con questo nome annulla
	if [file exists [file join ~ .correlatore report $nome]] {
		array set mess {it "Impossibile procedere: esiste già un report con questo nome." en "Cannot proceed: this report exists already"}
		tk_messageBox -parent $lb -message $mess($l)
		return
	}
	
	# se non è un txt annulla
	if {[file extension $nome] ne ".txt"} {
		array set mess {it "Impossibile procedere: il file di report deve avere estensione TXT." en "Cannot proceed: report files must have a TXT extension."}
		tk_messageBox -parent $lb -message $mess($l)
		return
	}
	
	# se tutto va bene aggiorna la lista dei report
	if [catch {file copy -force $file [file join ~ .correlatore report $nome]} error] {
		array set mess {it "Errore nell'esportazione:" en "An error has occurred in the exportation:"}
		tk_messageBox -parent $lb -message "$mess($l)\n$error"
	} else {
		set ::reportpath $nome
		wconfigure reportpath $nome
		grid [lista_report [winfo parent $lb]] -column 0 -row 1 -sticky news -padx 10
	}
}

proc esporta_report {lb} {
	global l
	set selezionato [$lb get [$lb curselection]]
	if {[set file [tk_getSaveFile -defaultextension .txt -initialfile $selezionato]] eq ""} {return}
	# se tutto va bene dai un messaggio di riuscita
	if [catch {file copy -force [file join ~ .correlatore report $selezionato] $file} error] {
		array set mess {it "Errore nell'esportazione:" en "An error has occurred in the exportation:"}
		tk_messageBox -parent $lb -message "$mess($l)\n$error"
	} else {
		array set mess {it " esportato nel file " en " has been successfully exported to "}
		tk_messageBox -parent $lb -message $selezionato$mess($l)$file
	}
}
